import axios, { AxiosResponse, Method } from "axios";
import qs from "qs";
import { getCookie, removeCookie } from "typescript-cookie";

export interface Session {
  user:string
  xsrf:string
  csrf:string
  authed_request:(verb:Method,url:string,data:any)=>Promise<any>
}


export const SITE_ROOT = "/lifeto/"
export const API_ROOT = "api/lifeto/"

const login_endpoint = (name:string)=>{
  return SITE_ROOT + name
}
const api_endpoint = (name:string)=>{
  return SITE_ROOT+API_ROOT + name
}


export class LoginHelper {
  user:string
  pass:string
  csrf?:string
  constructor(user:string, pass:string){
    this.user = user;
    this.pass = pass;
  }
  login = async ():Promise<TokenSession> =>{
    removeCookie("XSRF-TOKEN")
    await sleep(1000)
    return axios.get(login_endpoint("login"),{withCredentials:false})
    .then(async (x)=>{
      console.log(x)
      if(x.data){
        try{
          this.csrf = x.data.split("csrf-token")[1].split('\">')[0].replace("\" content=\"",'')
        }catch(e){
        }
      }
      return axios.post(login_endpoint("login"),{
        login:this.user,
        password:this.pass,
        redirectTo:"lifeto"
      },{withCredentials:false})
      .then(async (x)=>{
        await sleep(100)
        let xsrf= getCookie("XSRF-TOKEN")
        return new TokenSession(this.user,this.csrf!, xsrf!)
      })
    })
  }
}
const sleep= async(ms:number)=> {
  return new Promise(resolve => setTimeout(resolve, ms))
}

export class TokenSession implements Session {
  csrf:string
  xsrf:string
  user:string
  constructor(name:string, csrf:string, xsrf: string){
    this.user = name
    this.csrf = csrf
    this.xsrf = xsrf;
  }

  authed_request = async (verb:string,url:string,data:any):Promise<AxiosResponse> => {
    let promise
    switch (verb.toLowerCase()){
      case "post":
        promise = axios.post(api_endpoint(url),data,this.genHeaders())
      break;
      case "postraw":
        const querystring = qs.stringify(data)
      promise = axios.post(api_endpoint(url),querystring,this.genHeaders())
      break;
      case "get":
        default:
        promise = axios.get(api_endpoint(url),this.genHeaders())
    }
    return promise.then(x=>{
      if(x.data){
        try{
          this.xsrf = x.data.split("xsrf-token")[1].split('\">')[0].replace("\" content=\"",'')
        }catch(e){
        }
      }
      if(x.headers['set-cookie']){
        const cookies = x.headers['set-cookie'].map((y)=>{
          return y.split("=")[1].split(";")[0];
        })
        this.xsrf = cookies[0]
      }
      return x
    })
  }
  genHeaders = ()=>{
    const out = {
      headers:{
        Accept: "application/json",
        "Update-Insecure-Requests": 1,
      },
      withCredentials:true
    }
    if(this.xsrf){
      (out.headers as any)["X-XSRF-TOKEN"] = this.xsrf.replace("%3D","=")
    }
    return out
  }
}
